{% extends "layout.html" %}
{% from "macros/blog.html" import render_blog_post %}
{% from "macros/pagination.html" import render_pagination %}
{% block title %}{{ this.title }}{% endblock %}
{% block body %}
  {% for child in this.pagination.items %}
    {{ render_blog_post(child, from_index=true, blog_post=false) }}
  {% endfor %}

  {{ render_pagination(this.pagination) }}
{% endblock %}
