* {
    margin: 0;
}

html {
    height: 100%;
}

body {
    background-color: #fdfdfd;
    height: 100%;
    font-family: 'Open Sans', sans-serif;
    font-size: 1.5em;
    padding-top: 70px;
}

#green-terminal {
    color: #00ff00;
}

.font-header {
    font-family: 'Inconsolata', monospace;
}

.font-paragraph {
    font-family: 'Open Sans', sans-serif;
}

nav {
    font-size: 1.5em;
}


h1,h2,h3,h4,h5,h6 {
    font-family: 'Concert One', cursive;
}

#user-name {
    font-size: 5em;
}

.user-description {
    border-style: solid;
    border-width: 5px;
    border-color: #292929;
    margin-top: 20px;
    margin-bottom: 1em;
    padding: 15px;
    font-size: initial;
}

.user-profile {
    padding: 0;
}

.user-picture {
    border-style: solid;
    border-width: 5px;
    border-color: #292929;
    margin-top: 20px;
}

i {
    margin: 10px 20px 10px 20px;
    color: #333;
}

.post-link {
    float: left;
}

.post-date {
    float: right;
}

.post-header {
    background-color: #F5F5F5;
    overflow: hidden;
    padding: 0 5px 0 5px;
}

.post-summary, .post-content{
    clear: both;
}

.post-summary {
    padding: 0 5px 0 5px;
}

.post-list-footer {
    padding-bottom: 5px;
}

#post-list {
    list-style: none;
    padding-left: 0;
}

.post-list-item {
    margin-top: 20px;
    border-style: solid;
    border-width: 0 0 3px 0;
    border-color: #292929;
}

.post-comments {
    padding-top: 10px;
    border-top-style: solid;
}

.wrapper {
    min-height: 100%;
    margin: 0 auto -50px;
}

.footer {
    background-color: #F5F5F5;
    display: block;
    width: 100%;
    padding: 10px;
    height: 50px;
}

.push {
    height: 50px;
}

.col-centered{
    float: none;
    margin: 0 auto;
}

div p {
    text-align: justify;
}

td {
    margin: 0 20px 0 20px;
}
