/* Modal dialogs */

.dialog-slot {
  position: absolute;
  width: 100%;
}

.interface-protector {
  position: fixed;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  background: $protector-background;
  z-index: 1000;
}

.sliding-panel {
  position: relative;
  z-index: 5000;
  background: $background-taint;
  padding: 1rem 1.5rem;
  box-shadow: $panel-shadow;

  a.close-btn {
    float: right;
  }

  > :last-child {
    margin-bottom: 0;
  }
}

ul.search-results {
  list-style: none;
  margin: 0;
  padding: 0;

  li {
    margin: 2px 0;
    padding: 3px 10px;
    cursor: pointer;

    span:after {
      content: " / ";
    }
  }

  a {
    color: unset;
  }

  li.active,
  li:hover {
    background: white;
  }
}

.build-log {
  overflow: auto;
  max-height: 200px;
}

.interface-protector.server-down-panel {
  z-index: 10000;
}

.server-down-dialog {
  position: absolute;
  left: 50%;
  top: 50%;
  width: 360px;
  transform: translate(-50%, -50%);
  padding: 1rem;
  text-align: center;
  background: $background-taint;
  box-shadow: $panel-shadow;
}
