/* Fonts */

@font-face {
  font-family: "Roboto Slab";
  font-style: normal;
  font-weight: 300;
  src: local("Roboto Slab Light"), local("RobotoSlab-Light"),
    url("@openfonts/roboto-slab_all/files/roboto-slab-all-300.woff2")
      format("woff2"),
    url("@openfonts/roboto-slab_all/files/roboto-slab-all-300.woff")
      format("woff");
}

@font-face {
  font-family: "Roboto Slab";
  font-style: normal;
  font-weight: 400;
  src: local("Roboto Slab Regular"), local("RobotoSlab-Regular"),
    url("@openfonts/roboto-slab_all/files/roboto-slab-all-400.woff2")
      format("woff2"),
    url("@openfonts/roboto-slab_all/files/roboto-slab-all-400.woff")
      format("woff");
}
